#include <stdio.h>
#include <Wire.h>
#include <MadgwickAHRS.h>

#define DOF_9 1

Madgwick filter;
unsigned long microsPerReading, microsPrevious;
float accelScale, gyroScale;

int SERIAL_BAUDRATE = 2000000;

int GYROSCOPE_SENSITIVITY = 245; // 125/245/500/1000/2000
int ACCELEROMETER_SENSITIVITY = 4; // 2/4/8/16
int MAGNETOMETER_SENSITIVITY = 8; // 4/8/12/16


//Mag calibration settings (Gathered with the Pololu LIS3MDL Example)
int minX = -4327, maxX = 893;
int minY = -2712, maxY = 2374;
int minZ = -2842, maxZ = 2463;

float offsetX = (maxX + minX) / 2;
float offsetY = (maxY + minY) / 2;
float offsetZ = (maxZ + minZ) / 2;

float avgDelta = ((maxX - minX) + (maxY - minY) + (maxZ - minZ)) / 3;
float scaleX = avgDelta / (maxX - minX);
float scaleY = avgDelta / (maxY - minY);
float scaleZ = avgDelta / (maxZ - minZ);


//Gyro Calibration settings (Average gathered earlier using Python)
float offset_averageX = 540.4484484484484;
float offset_averageY = -515.6076076076076;
float offset_averageZ = -342.27927927927925;

float accel_readings[3];
float gyro_readings[3];
int imu_typed;

//Sensor number (for tests only)
int sensor_number = 4;

#if DOF_9 == 1
float magneto_readings[3];
#endif


void setup() {
  Serial.begin(2000000);
  Wire.begin();
  #if DOF_9 == 1
    SetupMagnetometer();
    imu_typed = 9;
  #else
    imu_typed = 6;
  #endif
  SetupIMU();
  filter.begin(25);

  microsPerReading = 1000000 / 25;
  microsPrevious = micros();
}


void loop() {
  int aix, aiy, aiz;
  int gix, giy, giz;
  float ax, ay, az;
  float gx, gy, gz;
  float roll, pitch, heading;
  unsigned long microsNow;

// check if it's time to read data and update the filter
  microsNow = micros();
  if (microsNow - microsPrevious >= microsPerReading) {

    #if DOF_9 == 1
      ReadMagneto();
    #endif
    ReadIMU();


    #if DOF_9 == 1
      filter.update(gyro_readings[0], gyro_readings[1], gyro_readings[2], accel_readings[0], accel_readings[1], accel_readings[2], magneto_readings[0], magneto_readings[1], magneto_readings[2]);

    #else
      filter.updateIMU(gyro_readings[0], gyro_readings[1], gyro_readings[2], accel_readings[0], accel_readings[1], accel_readings[2]);
 
    #endif
    Madgwick::Quaternion q = filter.getQuaternion();
    float q0 = q.q0;
    float q1 = q.q1;
    float q2 = q.q2;
    float q3 = q.q3;
    Serial.println('#'+String(sensor_number)+';'+String(9)+';'+String(q0)+';'+String(q1)+';'+String(q2)+";"+String(q3)+'/'+String(gyro_readings[0])+';'+String(gyro_readings[1])+';'+String(gyro_readings[2])+';'+String(accel_readings[0])+';'+String(accel_readings[1])+';'+String(accel_readings[2])+';'+String(magneto_readings[0])+';'+String(magneto_readings[1])+';'+String(magneto_readings[2]));// '!' = separator between sensors, '/' = comment (debugging readings i.e)

    // increment previous time, so we keep proper pace
    microsPrevious = microsPrevious + microsPerReading;
}}
